@@ -1,6 +1,50 @@
 # -*- coding:utf-8 -*-
 # Auther: Zed
+"""
+It's beautiful right? ^_^
 
+dm = Matrix()
+print('data_matrix:', dm)
+res = dm[dm['code'] == '000001']
+print('data_matrix_slice:\n', res)
+------------------------------
+data_matrix:
+             ['close', 'high', 'low', 'open', 'ret']
+100
+000001  [0.80676805 0.30489957...0.83385824 0.56749199]
+000002  [0.0158474  0.65408549...0.14293321 0.76696439]
+                  ......
+
+200
+000001  [0.62020139 0.96969446...0.76984185 0.14326731]
+000002  [0.82182821 0.10924971...0.9967727  0.51963606]
+                  ......
+                  ......
+400
+...
+000003  [0.15914842 0.99641071...0.15914842 0.99641071]
+Dimension: ['timestamp', 'code', 'features'], Shape: (4, 3, 5), dtype: matrix
+
+data_matrix_slice:
+         close      high       low      open       ret
+100  0.806768  0.304900  0.210918  0.833858  0.567492
+200  0.620201  0.969694  0.754948  0.769842  0.143267
+300  0.008385  0.087556  0.994873  0.706558  0.752703
+400  0.388965  0.946414  0.967339  0.567995  0.071485
+
+
+next plan:
+        3. Increase logic operation such as '>= <= > < !=/~ and & or / in' : dm[dm['datetime']>=20220105 and dm['code'] in ['000001','000002']]
+        4. Increase more __magic_function__ such as __del__: del dm["feature=='close'"] / dm.delete('datetime<=20220105')
+        5. Del dm[dm[::3]:ClassMatrix]
+        6. Increase function fill_na: dm.fill_na('forward_value/backward_value/zero_value/avg_value')
+        7. Query function: dm.query("20220105<=datetime<=20220506 and code in ['000001','000002'] and feature.capital>float(1e9)")
+        -1. More simular pd.DataFrame operation function in Matrix
+            * dm.sort(dm['feature']==close,ascend=True) / dm.sort('datetime')
+            * dm.resort_index('datetime') / dm.resort_value('datetime.20220510')
+                _datetime='datetime.20220510'.split('.')[1]
+                it will check dtype of dm.axis_dtype of datetime : data=float(_datetime) if self.axis_dict['datetime'].dtype==float else str(_datetime)
+"""
 from typing import Union, NewType
 from copy import deepcopy
 import numpy as np
@@ -102,6 +146,7 @@ def check_Series_unit(series):
     _type = None
     _type0 = None
     _type1 = None
+
     if type(series) == list:
         for i, value in enumerate(series):
 
@@ -123,6 +168,7 @@ def check_Series_unit(series):
                 if not _type:
                     _type = type(value)
                 else:
+
                     if _type != type(value):
                         raise ValueError(f'error type {series}')
 
@@ -218,6 +264,16 @@ _loc = NewType('_loc', dict)
 _SeriesDtype = Union[dict, tuple, str, int, float, list, _loc]
 
 
+# 最基本的元素
+class Unite:
+    def __init__(self, value, name):
+        self.value = value
+        self.name = name
+
+    def __str__(self):
+        print(f'name:{self.name}\n', self.value)
+
+
 class Series:
     def __init__(self, series: list, name: str = 'Undefined', unique=False, astype: str = None):
 
@@ -226,7 +282,7 @@ class Series:
 
         self.name = name
         self._series = series
-        self._series_value = []
+        self._series_value: np.ndarray
         self.length = len(self._series)
         self.index = list(range(self.length))
         self.unique = unique
@@ -239,6 +295,7 @@ class Series:
             self._series = list(zip(self.index, self._series_value))
 
         else:
+            # [('index_name0','value0'),('index_name1','value1')]
             _series_values = []
             _series_values_index = []
             for i, value in enumerate(self._series):
@@ -261,29 +318,66 @@ class Series:
     def values(self):
         return self._series_value
 
+    def _operator(self, sql):
+        _axis_value = np.where(sql)[0]
+        return Series(
+            _axis_value,
+            self.name, self.unique, 'loc.operator')
+
+    #
+    # def __contains__(self, other: list):
+    #     # dm['timestamp'] in [1,2,3,4,5]
+    #     # self.values[]
+
+    def __or__(self, other):
+        #     # or |
+
+        return Series([self, other], name=self.name, astype='loc.or_series')
+
+    def __ror__(self, other):
+        # or |
+
+        return Series([self, other], name=self.name, astype='loc.or_series')
+
+    def __and__(self, other):
+
+        return Series([self, other], name=self.name, astype='loc.and_series')
+
+    def __rand__(self, other):
+
+        return Series([self, other], name=self.name, astype='loc.and_series')
+
     def __eq__(self, other):
+        if type(other) == list:
+            res_list = []
+            for i in range(len(other)):
+                res = np.where(self.values == other[i])[0][0]
+                res_list.append(res)
 
-        _res = []
-        if self.unique:
-            for i, value in enumerate(self._series):
+            return Series(res_list, name=self.name, astype='loc.operator')
+        other = np.array(other)
+        sql = self._series_value == other
+        return self._operator(sql)
 
-                if value[1] == other:
-                    _res = [(self.name, i)]
-                    return Series(_res, 'loc.unique', True, 'loc')
-            if not _res:
-                _res = [(self.name, None)]
-                return Series(_res, 'loc.unique', True, 'loc')
-        else:
+    def __lt__(self, other):
+        sql = self._series_value < other
+        return self._operator(sql)
 
-            for value in self._series:
-                if value[1] == other:
-                    _res.append(True)
-                else:
-                    _res.append(False)
+    def __le__(self, other):
+        sql = self._series_value <= other
+        return self._operator(sql)
 
-            return Series(_res, self.name)
+    def __gt__(self, other):
+        sql = self._series_value > other
+        return self._operator(sql)
 
+    def __ge__(self, other):
+        sql = self._series_value >= other
+        return self._operator(sql)
 
+    def __ne__(self, other):
+        sql = self._series_value != other
+        return self._operator(sql)
 
     def __str__(self):
         _data = _print(self._series)
@@ -301,7 +395,7 @@ MatrixGetDataType = Union[np.ndarray, list, tuple, dict, None]
 
 
 class DataFrame:
-    def __init__(self, matrix_np: np.ndarray = None, axis_name: dict = None):
+    def __init__(self, matrix_np: np.ndarray = None, axis_name: dict = None, name='Undefined'):
         # matrix_np:二维np矩阵， axis_name：行列名
 
         if not axis_name or matrix_np is None:
@@ -313,6 +407,7 @@ class DataFrame:
         self.axis_name = axis_name
         self.index = self.axis_name['index']
         self.columns = self.axis_name['columns']
+        self.name = name
 
     def to_pandas_df(self) -> pd.DataFrame:
         _df = pd.DataFrame(self.data)
@@ -325,7 +420,7 @@ class DataFrame:
 
     def __str__(self):
         _df = self.to_pandas_df()
-        return str(_df)
+        return f'name:{self.name}\n' + str(_df)
 
     def __repr__(self):
         _df = self.to_pandas_df()
@@ -345,11 +440,16 @@ class Matrix:
     #            'features':[xx,xx,xx],
     #           }
     # 需要解决一个问题：如果两个dataframe的长度不一致时如何解决
-    def __init__(self, data: MatrixGetDataType = None, axis_name: _MatrixAxisNameType = None, unique=True):
-        # unique如果能确保值是唯一的，那么可提高性能
-
+    def __init__(self, data: MatrixGetDataType = None, axis_name: _MatrixAxisNameType = None, unique=True, sort=True,
+                 sort_axis=0, reverse=False):
+        # unique如果能确保轴值是唯一的，那么可提高性能
+        # sort=True：是否自动排序
+        # sort_axis=0：自动排序默认轴
+        # reverse：从小到大
         self._dimension_dict = {}  # 轴名为key，维度为value
         self._dimension_num_dict = {}  # 维度为key，轴名为value
+        self._dimension_axis_dict = {'index': 0, 'row': 1, 'columns': 2}
+        self._dimension_axis_num_dict = {0: 'index', 1: 'row', 2: 'columns'}
         self.unique = unique
 
         if type(data) == np.ndarray:
@@ -391,6 +491,9 @@ class Matrix:
         self._axis_name_key = []
         self._axis_value_dtype = {}
         self._axis_value_num_dtype = {}
+        self._is_sort = sort
+        self._sort_axis = sort_axis
+        self._reverse = reverse
         self.init()
 
     def T(self, axis_one, axis_two=None):
@@ -437,6 +540,7 @@ class Matrix:
         else:
             raise ValueError(f'error axis_num or axis_name: {axis_one, axis_two}')
         self._T(new_queue)
+
         return self
 
     def _T(self, new_queue):
@@ -448,6 +552,9 @@ class Matrix:
         self.axis_name = _axis_name_dict
         self.init()
 
+    def copy(self):
+        return deepcopy(self)
+
     def append(self, obj: _MatrixAppendObjType, name=None, axis_name=None):
         # axis_name:如果用户能输入轴名，那么就不需要作维度相等判断，因此性能会更高
         # 添加轴的轴value如datetime的value20220501
@@ -531,6 +638,8 @@ class Matrix:
                         f'The axis {key} type is not unified ({type(value)}!={self._axis_value_dtype[key]}')
 
     def init(self):
+        if self.data.size == 0:
+            return
         self.axis_value_type_check()
         if len(self.data.shape) != 3:
             raise ValueError(f'error data shape {self.data}')
@@ -560,6 +669,9 @@ class Matrix:
 
         self._axis_name_dict = _data_dict
         self._axis_name_key = list(self._axis_name_dict.keys())
+        if self._is_sort:
+            # 是否自动排序
+            self.sort(self._sort_axis, reverse=self._reverse)
 
     # ToDo 像df.loc一样定位数据
     @property
@@ -628,134 +740,248 @@ class Matrix:
     def __getitem__(self, *item):
 
         if len(item) == 1:
-            item = item[0]
+            if type(item[0]) in [str, int, slice, Series]:
+                item = item[0]
+                item: _MatrixGetSingleItemType
+                if type(item) == str:
+                    # dm['data_xx']:返回某维度所有index
+                    if item not in self._axis_name_key:
+                        raise ValueError(f'{item} not exist in Matrix')
 
-            item: _MatrixGetSingleItemType
-            if type(item) == str:
-                # dm['data_xx']:返回某维度所有index
-                if item not in self._axis_name_key:
-                    raise ValueError(f'{item} not exist in Matrix')
+                    return Series(list(self._axis_name_dict[item].keys()), item, self.unique)
+                elif type(item) == Series:
 
-                return Series(list(self._axis_name_dict[item].keys()), item, self.unique)
-            elif type(item) == Series:
+                    # 精确区间定位
+                    if item.dtype == 'loc.operator':
+                        _dimension = self._dimension_dict[item.name]
 
-                # Series类用于提供定位信息，根据dtype又分为bool和loc
-                if item.dtype == 'bool':
+                        _axis_name = self.axis_name
+                        _axis_name[item.name] = list(np.array(self.axis_name[item.name])[item.values])
 
-                    if item.name not in self._dimension_dict:
-                        raise ValueError(f'error {item.name} not in {self._dimension_dict}')
-                    _dimension = self._dimension_dict[item.name]
+                        if _dimension == 0:
+                            _data = self.data[item.values, :, :]
+                        elif _dimension == 1:
+                            _data = self.data[:, item.values, :]
+                        else:
+                            _data = self.data[:, :, item.values]
+
+                        return Matrix(_data, _axis_name, unique=self.unique)
+
+                        # 精确区间带步长定位
+                        # elif len(item.values) == 3:
+                        #     pass
+
+                    elif item.dtype == 'loc.in_series':
+
+                        # loc_in类型：适用于如dm[dm['code'] in [000001,000002,...,]]的
+                        # __in__ 定位指定样本
+                        # 这一步只是作为bool类型的代替，提高效率的方案
+                        _dimension = self._dimension_dict[item.index[0]]
+                        _axis_name = self.axis_name
+                        _axis_name[item.name] = list(np.array(_axis_name[item.name])[item.values])
+                        if _dimension == 0:
+                            _data = self.data[item.values, :, :]
+                        elif _dimension == 1:
+                            _data = self.data[:, item.values, :]
+                        else:
+                            _data = self.data[:, :, item.values]
+
+                        return Matrix(_data, _axis_name, unique=self.unique)
+
+                    elif item.dtype == 'loc.and_series':
+                        # 特别注意：如果multi_series里有返回dataframe的，必须要放在最后一个
+                        # 如'bool'类型和'loc.unite'类型
+
+                        new_matrix = self
+                        for _series in item.values:
+                            # print(new_matrix)
+                            if new_matrix.values.any():
+                                new_matrix = new_matrix[_series]
+                            else:
+                                return
+                        return new_matrix
+                        # 多组Series 的bool类型数据，如:
+                        #   0 Series([True,False]),name=code,dtype=bool
+                        #   1 Series([1234567,1234568,...]),name=timestamp,dtype=loc
+                        #  此时需要循环一步步定位，过滤数据
+                        # 常出现在dm[dm['timestamp']>1234567 and dm['code'] in ['000001','000002',...] or feature='close']使用场景
+                        # 因为and or in 的比较值Series DataFrame Matrix都有可能，所以都需要进行编写相关魔法函数
+                        #
+
+                    elif item.dtype == 'loc.or_series':
+
+                        for _series in item.values:
+
+                            new_matrix = deepcopy(self)
+
+                            new_matrix = new_matrix[_series]
+                            if new_matrix.values.any():
+                                return new_matrix
+
+                elif type(item) == slice:
+
+                    _data = self.values[item]
+                    item = 0
+                    _axis_name = deepcopy(self.axis_name)
+                    _axis_name.pop(self._dimension_num_dict[item])
+                    i = 0
+                    _df_name = {'index': [], 'columns': []}
+                    for key, value in _axis_name.items():
+                        if i == 0:
+                            _df_name['index'] = value
+                            i += 1
+                        else:
+                            _df_name['columns'] = value
+
+                    return DataFrame(_data[0], _df_name, name=str(self._dimension_num_dict[item]) + '==' + str(
+                        self.axis_name[self._dimension_num_dict[item]][0]))
+                elif type(item) == int:
+
+                    _data = self.values[item]
+
+                    _axis_name = deepcopy(self.axis_name)
+                    _axis_name.pop(self._dimension_num_dict[0])
+                    i = 0
+                    _df_name = {'index': [], 'columns': []}
+                    for key, value in _axis_name.items():
+                        if i == 0:
+                            _df_name['index'] = value
+                            i += 1
+                        else:
+                            _df_name['columns'] = value
+
+                    return DataFrame(_data, _df_name, name=str(self._dimension_num_dict[0]) + '==' + str(
+                        self.axis_name[self._dimension_num_dict[0]][item]))
+
+            elif len(item[0]) == 2:
+                if type(item[0][0]) == str:
+
+                    # support: dm['code',[000001,000002]], dm['code',1:10:2], dm['code',5]
+                    if item[0][0] in ['index', 'row', 'columns']:
+                        _dimension = self._dimension_axis_dict[item[0][0]]
+                    else:
+                        _dimension = self._dimension_dict[item[0][0]]
+                    _name = self._dimension_num_dict[_dimension]
+                    if type(item[0][1]) == slice:
+                        select_list = item[0][1]
+                    elif type(item[0][1]) == list:
+
+                        _axis_value = self.axis_name[_name]
+                        axis_value_list = []
+                        for i in range(len(item[0][1])):
+                            _res = np.where(np.array(_axis_value) == item[0][1][i])[0][0]
+                            axis_value_list.append(_res)
+                        select_list = axis_value_list
+                    elif type(item[0][1]) == int:
+                        select_list = item[0][1]
+                    else:
+                        _axis_value = self.axis_name[_name]
+                        axis_value_list = []
+                        _res = np.where(np.array(_axis_value) == item[0][1])[0][0]
+                        axis_value_list.append(_res)
+                        select_list = axis_value_list
 
                     if _dimension == 0:
-                        _res = []
-
-                        for i in range(len(item.values)):
-                            if item.values[i]:
-                                _res.append(self.data[i, :, :])
-                        _res = np.array(_res)
-                        _row_name = self._axis_name_key[1]
-                        _columns_name = self._axis_name_key[2]
-                        df_axis_name = dict(
-                            index=self.axis_name[_row_name],
-                            columns=self.axis_name[_columns_name]
-                        )
-
-                        return DataFrame(_res[0], df_axis_name)
-
+                        _value = self.values[select_list, :, :]
                     elif _dimension == 1:
-                        _res = []
-
-                        for i in range(len(item.values)):
-                            if item.values[i]:
-                                _res.append(self.data[:, i, :])
-                        _res = np.array(_res)
-                        _row_name = self._axis_name_key[0]
-                        _columns_name = self._axis_name_key[2]
-                        df_axis_name = dict(
-                            index=self.axis_name[_row_name],
-                            columns=self.axis_name[_columns_name]
-                        )
-
-                        return DataFrame(_res[0], df_axis_name)
+                        _value = self.values[:, select_list, :]
                     else:
-                        _res = []
+                        _value = self.values[:, :, select_list]
 
-                        for i in range(len(item.values)):
-                            if item.values[i]:
-                                _res.append(self.data[:, :, i])
-                        _res = np.array(_res)
+                    if type(item[0][1]) == int:
+                        _axis_name = deepcopy(self.axis_name)
+                        _axis_name.pop(_name)
+                        new_axis = {}
+                        _i = 0
+                        # print(_axis_name)
+                        for _key0, _value0 in _axis_name.items():
+                            if _i == 0:
+                                new_axis['index'] = _value0
+                                _i += 1
+                            else:
+                                new_axis['columns'] = _value0
 
-                        _row_name = self._axis_name_key[0]
-                        _columns_name = self._axis_name_key[1]
-                        df_axis_name = dict(
-                            index=self.axis_name[_row_name],
-                            columns=self.axis_name[_columns_name]
-                        )
+                        return DataFrame(_value, axis_name=new_axis,
+                                         name=_name + '==' + str(self.axis_name[_name][select_list]))
+                    _axis_value = self.axis_name[_name]
+                    _axis_value = np.array(_axis_value)
 
-                        return DataFrame(_res[0], df_axis_name)
-                elif item.dtype == 'loc':
+                    _axis_value = list(_axis_value[select_list])
+                    self.axis_name[_name] = _axis_value
 
-                    if not len(item.values):
-                        return
-                    if len(item.values) == 1:
+                    return Matrix(_value, self.axis_name, unique=self.unique)
 
-                        _dimension = self._dimension_dict[item.index[0]]
-                        if _dimension == 0:
-                            _res = [self.data[item.values[0], :, :]]
+                elif type(item[0][0]) == int:
+                    if type(item[0][1]) == int:
+                        raise ValueError(f'Unsupported at least 2 int type params: {item[0]}')
+                        # _value = self.values[item[0][0], type(item[0][1])]
+                        # print(_value)
+                    elif type(item[0][1]) == slice:
+                        # ToDO 要么只有slice，那么只有一个int
+                        raise ValueError(f'Unsupported 1 int type  and 1 slice type params: {item[0]}')
+                    else:
+                        raise ValueError(f'Unsupported  params: {item[0]}')
 
-                            _res = np.array(_res)
-                            _row_name = self._axis_name_key[1]
-                            _columns_name = self._axis_name_key[2]
-                            df_axis_name = dict(
-                                index=self.axis_name[_row_name],
-                                columns=self.axis_name[_columns_name]
-                            )
+                elif type(item[0][0]) == slice:
 
-                            return DataFrame(_res[0], df_axis_name)
+                    if type(item[0][1]) == int:
+                        raise ValueError(f'Unsupported  params: {item[0]}')
+                    elif type(item[0][1]) == slice:
+                        # 支持双切割 dm[2:10:1,5:10:2]
+                        _value = self.data[item[0][0], item[0][1]]
+                        i = 0
+                        _axis_name = {}
+                        _axis_name_copy = deepcopy(self.axis_name)
 
-                        elif _dimension == 1:
-                            _res = [self.data[:, item.values[0], :]]
+                        for key, value in _axis_name_copy.items():
+                            if i == 0:
+                                _axis_name[key] = list(np.array(value)[item[0][0]])
 
-                            _res = np.array(_res)
-                            _row_name = self._axis_name_key[0]
-                            _columns_name = self._axis_name_key[2]
-                            df_axis_name = dict(
-                                index=self.axis_name[_row_name],
-                                columns=self.axis_name[_columns_name]
-                            )
+                            elif i == 1:
+                                _axis_name[key] = list(np.array(value)[item[0][1]])
+                            else:
+                                _axis_name[key] = list(np.array(value))
+                            i += 1
 
-                            return DataFrame(_res[0], df_axis_name)
+                        return Matrix(_value, _axis_name, unique=self.unique)
+                    else:
+                        raise ValueError(f'Unsupported  params: {item[0]}')
 
+
+                else:
+                    raise ValueError(f'Unsupported  params: {item[0]}')
+
+            elif len(item[0]) == 3:
+                if type(item[0][0]) == slice and type(item[0][1]) == slice and type(item[0][2]) == slice:
+                    _value = self.data[item[0][0], item[0][1], item[0][2]]
+                    i = 0
+                    _axis_name = {}
+                    _axis_name_copy = deepcopy(self.axis_name)
+
+                    for key, value in _axis_name_copy.items():
+                        if i == 0:
+                            _axis_name[key] = list(np.array(value)[item[0][0]])
+
+                        elif i == 1:
+                            _axis_name[key] = list(np.array(value)[item[0][1]])
                         else:
+                            _axis_name[key] = list(np.array(value)[item[0][2]])
+                        i += 1
 
-                            _res = [self.data[:, :, item.values[0]]]
-
-                            _res = np.array(_res)
-                            _row_name = self._axis_name_key[0]
-                            _columns_name = self._axis_name_key[1]
-                            df_axis_name = dict(
-                                index=self.axis_name[_row_name],
-                                columns=self.axis_name[_columns_name]
-                            )
-
-                            return DataFrame(_res[0], df_axis_name)
-
-            elif type(item) == list:
-
-                if self.unique:
-                    return
+                    return Matrix(_value, _axis_name, unique=self.unique)
+                else:
+                    raise ValueError(f'Unsupported  params: {item[0]}')
 
         else:
+            raise ValueError(f'Unsupported  params: {item[0]}')
 
-            print('_MatrixGetMultiItemType')
-            pass
 
     def to_pkl(self, save_name: str):
         to_pkl(self, save_name)
 
     def to_df(self, is_matrix_index=True, matrix_index_split='.'):
-        _df = to_pandas(self, is_matrix_index=is_matrix_index, matrix_index_split=matrix_index_split)
-        return _df
+
+        return to_pandas(self, is_matrix_index=is_matrix_index, matrix_index_split=matrix_index_split)
 
     @property
     def values(self):
@@ -801,6 +1027,8 @@ class Matrix:
         return _print_data
 
     def __str__(self):
+        if self.data.size == 0:
+            return 'None'
         return self._print()
 
 
@@ -1002,6 +1230,14 @@ def read_pandas(obj: pd.DataFrame, shape=None, axis_name: dict = None):
 
 
 def to_pandas(obj: Matrix, is_matrix_index: bool = True, matrix_index_split: str = '.') -> pd.DataFrame:
+    # ToDo 直接将三维转为标准的二维df
+    # 检查哪个是一条数据，就能确定
+    # _axis_name_value = list(obj.axis_name.values())
+    # _dimension = 0
+    # for i in range(len(_axis_name_value)):
+    #     if len(_axis_name_value[i]) == 1:
+    #         _dimension = i
+
     new_dm_data = obj.data.reshape([obj.data.shape[0] * obj.data.shape[1], obj.data.shape[2]])
     _index = obj.index
     _row = obj.row
@@ -1026,4 +1262,17 @@ def to_pandas(obj: Matrix, is_matrix_index: bool = True, matrix_index_split: str
     return df
 
 
+def matrix_fillna(arr: np.ndarray):
+    # 目前适用于二维矩阵
+    mask = np.isnan(arr)
+    idx = np.where(~mask, np.arange(mask.shape[1]), 0)
+    np.maximum.accumulate(idx, axis=1, out=idx)
+    arr[mask] = arr[np.nonzero(mask)[0], idx[mask]]
+    return arr
+
+def to_std_df():
+    # 转为完美的pandas.DataFrame
+    pass
+
+
 
